@@ -2,6 +2,7 @@ import datetime as dt
 import inspect
 import os
 import posixpath
+import threading
 from datetime import datetime, timedelta
 from functools import wraps
 from pathlib import Path
@@ -25,18 +26,42 @@ from fsspec.implementations.dirfs import DirFileSystem
 from loguru import logger
 
 from .helpers.misc import read_table, run_parallel
+from .helpers.security import safe_join, validate_path
 from .schema import convert_large_types_to_normal
 
 
-def get_credentials_from_fssspec(fs: AbstractFileSystem) -> dict[str, str]:
+def get_credentials_from_fssspec(fs: AbstractFileSystem, redact_secrets: bool = True) -> dict[str, str]:
+    """
+    Safely extract credentials from fsspec filesystem.
+    
+    Args:
+        fs: The fsspec filesystem object
+        redact_secrets: If True, redact sensitive values from returned dict
+        
+    Returns:
+        Dictionary with credential information (secrets redacted if redact_secrets=True)
+    """
     if "s3" in fs.protocol:
-        credendials = fs.s3._get_credentials()
-        return {
-            "access_key": credendials.access_key,
-            "secret_key": credendials.secret_key,
-            "session_token": credendials.token,
-            "endpoint_override": fs.s3._endpoint.host,
-        }
+        credentials = fs.s3._get_credentials()
+        
+        if redact_secrets:
+            # For logging and display purposes, redact actual secrets
+            return {
+                "access_key": f"REDACTED({len(credentials.access_key) if credentials.access_key else 0} chars)",
+                "secret_key": f"REDACTED({len(credentials.secret_key) if credentials.secret_key else 0} chars)",
+                "session_token": f"REDACTED({len(credentials.token) if credentials.token else 0} chars)" if credentials.token else None,
+                "endpoint_override": fs.s3._endpoint.host if fs.s3._endpoint else None,
+            }
+        else:
+            # Only return actual credentials when explicitly requested
+            return {
+                "access_key": credentials.access_key,
+                "secret_key": credentials.secret_key,
+                "session_token": credentials.token,
+                "endpoint_override": fs.s3._endpoint.host if fs.s3._endpoint else None,
+            }
+    
+    return {}
 
 
 def get_total_directory_size(directory: str):
@@ -45,13 +70,22 @@ def get_total_directory_size(directory: str):
 
 class FileNameCacheMapper(AbstractCacheMapper):
     def __init__(self, directory):
-        self.directory = directory
+        # Validate and normalize the directory path
+        self.directory = validate_path(directory)
 
     def __call__(self, path: str) -> str:
-        os.makedirs(
-            posixpath.dirname(posixpath.join(self.directory, path)), exist_ok=True
-        )
-        return path
+        # Validate the input path to prevent traversal
+        validated_path = validate_path(path)
+        
+        # Safely join paths
+        full_path = safe_join(self.directory, validated_path)
+        
+        # Create parent directory if needed
+        parent_dir = posixpath.dirname(full_path)
+        os.makedirs(parent_dir, exist_ok=True)
+        
+        # Return relative path from directory
+        return validated_path
 
 
 class throttle(object):
@@ -89,28 +123,47 @@ def sizeof_fmt(num, suffix="B"):
     return f"{num:.1f}Yi{suffix}"
 
 
-last_free = None
-first_free = None
+class DiskUsageTracker:
+    """Thread-safe disk usage tracking."""
+    
+    def __init__(self):
+        self._last_free = None
+        self._first_free = None
+        self._lock = threading.Lock()
+    
+    def get_usage_message(self, storage: str) -> str:
+        """Get friendly disk usage message with deltas."""
+        with self._lock:
+            usage = psutil.disk_usage(storage)
+            if self._first_free is None:
+                self._first_free = usage.free
+            current_usage = get_total_directory_size(storage)
+            message = f"{sizeof_fmt(current_usage)} used {sizeof_fmt(usage.free)} available"
+            if self._last_free is not None:
+                downloaded_recently = self._last_free - usage.free
+                if downloaded_recently > 10_000_000:
+                    downloaded_since_start = self._first_free - usage.free
+                    if downloaded_recently != downloaded_since_start:
+                        message += f" delta: {sizeof_fmt(downloaded_recently)}"
+                    message += f" delta since start: {sizeof_fmt(downloaded_since_start)}"
+
+            self._last_free = usage.free
+            return message
+    
+    def reset(self):
+        """Reset tracking state."""
+        with self._lock:
+            self._first_free = None
+            self._last_free = None
+
+
+# Global instance for backward compatibility
+_disk_tracker = DiskUsageTracker()
 
 
 def get_friendly_disk_usage(storage: str) -> str:
-    global last_free
-    global first_free
-    usage = psutil.disk_usage(storage)
-    if first_free is None:
-        first_free = usage.free
-    current_usage = get_total_directory_size(storage)
-    message = f"{sizeof_fmt(current_usage)} used {sizeof_fmt(usage.free)} available"
-    if last_free is not None:
-        downloaded_recently = last_free - usage.free
-        if downloaded_recently > 10_000_000:
-            downloaded_since_start = first_free - usage.free
-            if downloaded_recently != downloaded_since_start:
-                message += f" delta: {sizeof_fmt(downloaded_recently)}"
-            message += f" delta since start: {sizeof_fmt(downloaded_since_start)}"
-
-    last_free = usage.free
-    return message
+    """Get friendly disk usage message (legacy function)."""
+    return _disk_tracker.get_usage_message(storage)
 
 
 #############################################################################################################
@@ -935,7 +988,7 @@ def PyArrowFileSystem(
             protocol = fs.protocol[0] if isinstance(fs.protocol, tuple) else fs.protocol
 
         if protocol == "s3":
-            credentials = get_credentials_from_fssspec(fs)
+            credentials = get_credentials_from_fssspec(fs, redact_secrets=False)
 
     if credentials is None:
         credentials = {
